unit Unit1;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, Menus, ExtCtrls, EditPlugAPI, EditPlugAPITypes;

type
  TForm1 = class(TForm)
    Panel2: TPanel;
    Label1: TLabel;
    Panel3: TPanel;
    ListBox: TListBox;
    Popup: TPopupMenu;
    Modify1: TMenuItem;
    New1: TMenuItem;
    Insert1: TMenuItem;
    Remove1: TMenuItem;
    procedure FormShow(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure Insert1Click(Sender: TObject);
    procedure ListBoxDblClick(Sender: TObject);
    procedure Modify1Click(Sender: TObject);
    procedure New1Click(Sender: TObject);
    procedure Remove1Click(Sender: TObject);
    procedure ListBoxMouseDown(Sender: TObject; Button: TMouseButton;
      Shift: TShiftState; X, Y: Integer);
  private
    { Private declarations }
    FirstShow: boolean;
    procedure Insert(item: string);
  public
    { Public declarations }
    procedure LoadClips;
  end;

function ClipFilePath(item: string): string;

var
  Form1: TForm1;
  gPlugId: HINST;

implementation

{$R *.DFM}

uses
  StringUtils, fModify, uAppData;

{----- EXPORTS ----------------------------------------------------------------}

procedure plgSetId(id: HINST); cdecl;
begin
  gPlugId:= id;
end; exports plgSetId;

function plgShow: HWND; cdecl;
begin
  if not assigned(form1) then begin
    form1:= tform1.create(nil);
    form1.show;
    epCreatePluginTab(gPlugId, form1.handle);
  end else begin
    form1.show;
  end;
  result:= form1.Handle;
end; exports plgShow;

procedure plgWinFree; cdecl;
begin
  if assigned(form1) then begin
    form1.Free;
    form1:=nil;
  end;

  if assigned(fmModify) then begin
    fmModify.free;
    fmModify := nil;
  end;
end; exports plgWinFree;

function plgGetWinHandle: HWND; cdecl;
begin
  result:=0;
  if assigned(form1) then result:= form1.Handle;
end; exports plgGetWinHandle;

{------------------------------------------------------------------------------}

function ProcessEscapeChars(s: string): string;
begin
  result :=  StringReplace(s, '^^', '^', [rfReplaceAll]);
end;

function ClipFilePath(item: string): string;
begin
  //  result := ExtractFilePath(LibPath) + 'clips\' + item;
  result := IncludeTrailingBackslash(GetAppdataFolder) + 'EditPlug\';
  CreateDir(result);
  result := result + 'plugins\';
  CreateDir(result);
  result:= result + 'ClipText\';
  CreateDir(result);
  result:= result + 'clips\';
  CreateDir(result);
  result := result + item;
end;

function ClipDirPath: string;
begin
  result := IncludeTrailingBackslash(GetAppdataFolder) + 'EditPlug\';
  CreateDir(result);
  result := result + 'plugins\';
  CreateDir(result);
  result:= result + 'ClipText\';
  CreateDir(result);
  result:= result + 'clips\';
  CreateDir(result);
end;

(*
// problem when using stringlist.. adds carriage return to end of file
function File2Str(fname: string): string;
var sl: TStringList;
begin
  result := '';
  if not FileExists(fname) then exit;
  sl := TStringList.create;
  try
    sl.loadfromfile(fname);
    result := sl.text;
  finally
    sl.free;
  end;
end;

*)

function File2Str(fname: string): string;
var
  FileStream : TFileStream;
begin
  FileStream:= TFileStream.Create(fname, fmOpenRead or fmShareDenyWrite);
  try
    if FileStream.Size>0 then begin
      SetLength(Result, FileStream.Size);
      FileStream.Read(Pointer(Result)^, FileStream.Size);
    end;
  finally
    FileStream.Free;
  end;
end;

procedure TForm1.Insert(item: string);
var
  str: string;
begin
  if epGetEdCount < 1 then exit;
  str := File2Str(ClipFilePath(item));
  str := StringReplace(str, '^^', #255, [rfReplaceAll]);
  str := StringReplace(str, '^!', epGetActiveEdSelTextAsString, [rfReplaceAll]);
  str := StringReplace(str, #255, '^', [rfReplaceAll]);
  epSetActiveEdSelText(pchar(str));
  epSetActiveEdFocus;
end;

procedure TForm1.LoadClips;
var
  searchResult : TSearchRec;
  path: string;
begin
  ListBox.Clear;
  path := ClipDirPath;
  if FindFirst(path + '*', faAnyFile-faDirectory, searchResult) = 0 then
  begin
    repeat
        ListBox.Items.add(searchResult.Name);
    until FindNext(searchResult) <> 0;
    FindClose(searchResult);
  end;
end;

procedure TForm1.FormShow(Sender: TObject);
begin
  if FirstShow = true then begin
    FirstShow := false;
    LoadClips;
  end;
  if ListBox.ItemIndex < 0 then listbox.itemindex := 0;
end;

procedure TForm1.FormCreate(Sender: TObject);
begin
  FirstShow := true;
end;

procedure TForm1.Insert1Click(Sender: TObject);
begin
  self.Insert(ListBox.Items[listbox.ItemIndex]);
end;

procedure TForm1.ListBoxDblClick(Sender: TObject);
begin
  self.Insert(ListBox.Items[listbox.ItemIndex]);
end;

procedure ShowFmModify;
begin
  if not assigned(fmModify) then begin
    fmModify := TfmModify.create(nil);
    epSetParentToMainWin(fmModify.handle);
    fmModify.show;
    epMoveWinToTopRight(fmModify);
  end else
    fmModify.show;
end;

procedure TForm1.Modify1Click(Sender: TObject);
begin
  if listbox.ItemIndex < 0 then begin
    showmessage('Select a clip to modify.');
    exit;
  end;
  ShowFmModify;
  fmModify.Caption := 'Modify Clip';
  fmModify.edName.Text := ListBox.Items[listbox.ItemIndex];
  fmModify.edName.Enabled := false;
  fmModify.memo.Lines.LoadFromFile(ClipFilePath(fmModify.edName.Text));
end;

procedure TForm1.New1Click(Sender: TObject);
begin
  ShowFmModify;
  fmModify.Caption := 'Add Clip';
  fmModify.edName.Enabled := true;
  fmModify.edName.Text := '';
  fmModify.memo.Text := '';
end;

procedure DeleteFile(fname: string);
begin
  if not sysutils.DeleteFile(fname) then
    showmessage('Could not delete file: '+fname + ' Error: ' + IntToStr(GetLastError));
  Form1.LoadClips;
end;

procedure TForm1.Remove1Click(Sender: TObject);
begin
  if listbox.itemindex < 0 then begin
    showmessage('Select a clip to remove');
    exit;
  end;
  DeleteFile(ClipFilePath(listbox.Items[listbox.itemindex]));
end;

procedure TForm1.ListBoxMouseDown(Sender: TObject; Button: TMouseButton;
  Shift: TShiftState; X, Y: Integer);
var
  APoint: TPoint;
  Index: integer;
begin
  if Button = mbRight then begin
    APoint.X := X;
    APoint.Y := Y;
    Index := ListBox.ItemAtPos(APoint, True);
    if Index > -1 then
      ListBox.ItemIndex := index;
  end;
end;

initialization

finalization

end.
